/*
Example MQTT-switch-relay-node with 4 buttons and 4 leds

    connects to an MQTT server

    publishes “hello world" to the topic “led"

    subscribes to the topic “led"

    controls 4 leds on pins 2,3,5 and 6 - leds can be replaced with relays

    reads 4 button on pins 7,8,9 and 10

    turns on/off a specific led when it receives a specific “on"/“off" from the “led" topic

    sends a specific “on"/“off" to the “led" topic a specific button is pressed

    multiple arduino’s with same generic sketch can run parallel to each other

    multiple arduino’s need each to have a unique ip-addres, unique mac address and unique MQTT client-ID

    tested on arduino-uno with W5100 ethernet shield

    Ethernet Shield W5100 uses pins 4,10,11,12,13

    availbale digital pins: 1,2,3,5,6,7,8,9,10
    */

//------------------------------------------------------------------------------

#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>


// Set led variables to Arduino digital pins
const byte relay1 = 2;
const byte relay2 = 3;
const byte relay3 = 5; // pin 4 used by ethernet shield
const byte relay4 = 6;
const byte relay5 = 7;
const byte relay6 = 8;
const byte relay7 = 9;
const byte relay8 = 10;



//---------------------------------------------------------------------------

// Arduino MAC address is on a sticker on your Ethernet shield
// must be unique for every node in same network
// To make a new unique address change last letter

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xEE };

// Unique static IP address of this Arduino - change to adapt to your network
IPAddress ip(192,168,0,238);

// IP Address of your MQTT broker - change to adapt to your network
byte server[] = { 192, 168, 0, 20 };

// Handle and convert incoming MQTT messages ----------------------------------------

void callback(char*, byte*, unsigned int);



// Initiate instances -----------------------------------

EthernetClient ethClient;
PubSubClient client(server, 1883, callback, ethClient);



//-------------------------------------------------------

void setup()

{

// setup relays
    pinMode(relay1, OUTPUT);
    pinMode(relay2, OUTPUT);
    pinMode(relay3, OUTPUT);
    pinMode(relay4, OUTPUT);
    pinMode(relay5, OUTPUT);
    pinMode(relay6, OUTPUT);
    pinMode(relay7, OUTPUT);
    pinMode(relay8, OUTPUT);






// setup serial and ethernet communications -------------------------------

// Setup serial connection
Serial.begin(9600);

// Setup ethernet connection to MQTT broker
Ethernet.begin(mac);
    if (client.connect("arduino-ip-238")) { // change as desired - clientname must be unique for MQTT broker
        client.publish("status/relay","hello world - here arduino ip 239");
        Serial.println("connected");
        client.subscribe("control/relay"); // subscribe to topic “led"
    }
}

//----------------------------------------------

void loop()
{
    client.loop();
}

void callback(char* topic, byte* payload, unsigned int length) {
    // handle message arrived
    String content="";
    char character;

    for (int num=0;num<length;num++) {
        character = payload[num];
        content.concat(character);
    }

    Serial.println(topic);
    Serial.println(content); // message sent out by button actions is return

    // Set specific virtual switches on basis of specific incoming messages 

    if (content == "1on") {
        digitalWrite(relay1,HIGH);
        client.publish("state/relay","1on");
    }

    if (content == "1off") {
        digitalWrite(relay1, LOW);
        client.publish("state/relay","1off");
    }

    if (content == "2on") {
        digitalWrite(relay2, HIGH);
        client.publish("state/relay","2on");
    }

    if (content == "2off") {
        digitalWrite(relay2, LOW);
        client.publish("state/relay","2off");
    }

    if (content == "3on") {
        digitalWrite(relay3, HIGH);
        client.publish("state/relay","3on");
    }

    if (content == "3off") {
        digitalWrite(relay3, LOW);
        client.publish("state/relay","3off");
    }

    if (content == "4on") {
        digitalWrite(relay4, HIGH);
        client.publish("state/relay","4on");
    }

    if (content == "4off") {
        digitalWrite(relay4, LOW);
        client.publish("state/relay","4off");
    }

    if (content == "5on") {
        digitalWrite(relay5, HIGH);
        client.publish("state/relay","5on");
    }

    if (content == "5off") {
        digitalWrite(relay5, LOW);
        client.publish("state/relay","5off");
    }

    if (content == "6on") {
        digitalWrite(relay6, HIGH);
        client.publish("state/relay","6on");
    }

    if (content == "6off") {
        digitalWrite(relay6, LOW);
        client.publish("state/relay","6off");
    }

    if (content == "7on") {
        digitalWrite(relay7, HIGH);
        client.publish("state/relay","7on");
    }

    if (content == "7off") {
        digitalWrite(relay7, LOW);
        client.publish("state/relay","7off");
    }

    if (content == "8on") {
        digitalWrite(relay8, HIGH);
        client.publish("state/relay","8on");
    }

    if (content == "8off") {
        digitalWrite(relay8, LOW);
        client.publish("state/relay","8off");
    }

    
}

// End of sketch ---------------------------------
